package org.mortbay.jetty.toolchain.eclipse.signing;

//========================================================================
//Copyright (c) 2010 Intalio, Inc.
//------------------------------------------------------------------------
//All rights reserved. This program and the accompanying materials
//are made available under the terms of the Eclipse Public License v1.0
//and Apache License v2.0 which accompanies this distribution.
//The Eclipse Public License is available at 
//http://www.eclipse.org/legal/epl-v10.html
//The Apache License v2.0 is available at
//http://www.opensource.org/licenses/apache2.0.php
//You may elect to redistribute this code under either of these licenses. 
//========================================================================

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.codehaus.plexus.archiver.zip.ZipArchiver;
import org.codehaus.plexus.archiver.zip.ZipUnArchiver;
import org.codehaus.plexus.util.FileUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * 
 * @goal fixCheckSums
 * @phase package
 * @description updates the md5 checksum and file size
 */
public class ChecksumMojo extends AbstractEclipseSigningMojo
{
    /**
     * zip file to get checksum'ed
     * 
     * @parameter default-value="${project.build.directory}/${project.artifactId}-${project.version}.zip"
     * @required
     */
    protected String inputFile;

    /**
     * zip file to get checksum'ed
     * 
     * @parameter default-value="${project.build.directory}/fixed/${project.artifactId}-${project.version}.zip"
     * @required
     */
    protected String outputFile;

    /**
     * zip file to get checksum'ed
     * 
     * @parameter default-value="${project.build.directory}/checksumFix"
     * @required
     */
    protected String unzipDir;

    /**
     * unpack path for artifact
     * 
     * @parameter default-value="${project.artifactId}-${project.version}"
     * @required
     */
    protected String unpackPath;

    /**
     * checksum file
     * 
     * @parameter default-value="${project.build.directory}/${project.artifactId}-${project.version}/site/artifacts.xml
     * @required
     */
    protected String artifactsXml;

    public void execute() throws MojoExecutionException, MojoFailureException
    {
        try
        {

            if (!FileUtils.fileExists(unzipDir))
            {
                info("creating directory to hold unzipped artifact");
                FileUtils.mkdir(unzipDir);
            }
            
            if (!FileUtils.fileExists(FileUtils.dirname(outputFile)))
            {
                info("creating directory to hold output");
                FileUtils.mkdir(FileUtils.dirname(outputFile));
            }

            ZipUnArchiver unzipper = new ZipUnArchiver(new File(inputFile));
            unzipper.extract(unpackPath,new File(unzipDir));

            List files = FileUtils.getFileNames(new File(unzipDir),"**/*.jar","",true);

            for (Iterator i = files.iterator(); i.hasNext();)
            {
                String filename = (String)i.next();
                info("fixing " + filename);
                update(filename);
            }

            ZipArchiver zipper = new ZipArchiver();

            zipper.addDirectory(new File(unzipDir));
            zipper.setDestFile(new File(outputFile));
            zipper.createArchive();

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

    }

    private void update(String plugin)
    {
        try
        {
            String pluginFile = FileUtils.basename(plugin,".jar");

            String[] artifactBits = pluginFile.split("_");

            getLog().info(artifactBits[0] + "/" + artifactBits[1]);

            File inFile = new File(plugin);
            String inFileSize = Long.toString(inFile.length());

            int count = 0;
            byte[] buffer = new byte[1024 * 16];
            InputStream in = new FileInputStream(inFile);
            MessageDigest md = MessageDigest.getInstance("MD5");

            while ((count = in.read(buffer)) > 0)
            {
                md.update(buffer,0,count);
            }
            in.close();

            String inFileMD5 = convert(md.digest());// (new BigInteger(1, md.digest())).toString(16);

            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document document = builder.parse(new File(artifactsXml));

            String expr;
            XPath xpath = XPathFactory.newInstance().newXPath();

            expr = "/repository/artifacts//artifact[@id='" + artifactBits[0] + "' and @version='" + artifactBits[1]
                    + "']/properties//property[@name='artifact.size']/@value";
            Node artifactSize = (Node)xpath.evaluate(expr,document,XPathConstants.NODE);
            if (artifactSize != null)
            {
                artifactSize.setNodeValue(inFileSize);
            }

            expr = "/repository/artifacts//artifact[@id='" + artifactBits[0] + "' and @version='" + artifactBits[1]
                    + "']/properties//property[@name='download.size']/@value";
            Node downloadSize = (Node)xpath.evaluate(expr,document,XPathConstants.NODE);
            if (downloadSize != null)
            {
                downloadSize.setNodeValue(inFileSize);
            }

            expr = "/repository/artifacts//artifact[@id='" + artifactBits[0] + "' and @version='" + artifactBits[1]
                    + "']/properties//property[@name='download.md5']/@value";
            Node downloadMD5 = (Node)xpath.evaluate(expr,document,XPathConstants.NODE);
            if (downloadMD5 != null)
            {
                downloadMD5.setNodeValue(inFileMD5);
            }

            Transformer txformer = TransformerFactory.newInstance().newTransformer();
            DOMSource src = new DOMSource(document);
            StreamResult res = new StreamResult(artifactsXml);
            txformer.transform(src,res);
        }
        catch (Exception e)
        {
            getLog().error(e);
        }
    }

    private String convert(byte[] bytes)
    {
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < bytes.length; ++i)
        {
            sb.append(Character.forDigit((bytes[i] >> 4) & 0xf,16));
            sb.append(Character.forDigit(bytes[i] & 0xf,16));
        }
        return sb.toString();
    }
    
    private void info(String log)
    {
        getLog().info("[FIX] " + log);
    }

}
